      REAL FUNCTION o3column(dens, nlyr, zd) 
*
*BOD
*DESCRIPTION
* Scales the ozone profile dens to ozone_column.
*USAGE
**FORTRAN
*     CALL scale_ozone(dens, nlyr, ozone_column, zd) 
**PERL
*INPUT
**dens
*     real array(*)
*     Ozone density in number of molecules per cubic centimeter.
**nlyr
*     integer
*     Number of atmospheric layers.
**ozone_column
*     real
*     The ozone column in DU (Doboson units). dens will be scaled 
*     to ozone_column.
**zd
*     real array(*)
*     The altitude of each level.
*OUTPUT
**dens
*     real array(*)
*     Ozone density in number of molecules per cubic centimeter
*     scaled to ozone_column.
*ERROR_MESSAGES
*ROUTINES_CALLED
*FILES
*REFERENCES
*EOD
*
      IMPLICIT LOGICAL (A-Z)    ! To nearly get strong typing
*
      INTEGER lc, nlyr
      REAL dens(0:*), zd(0:*)
      REAL colo3, deltaz

*
* Calculate the ozone column in dobson
*
      colo3 = 0.0
      DO lc = 1, nlyr
         deltaz = ( zd(lc-1)-zd(lc) ) * 1.E+5 ! Convert from km to cm
         colo3 = colo3 + 0.5 * (dens(lc-1)+dens(lc)) * deltaz
      ENDDO
      o3column = colo3 / 2.687E+16 ! Convert to dobson units
*
      END

