/* lex.l                                                -*- C -*-

Copyright (C) 1995 Arve Kylling

This file is part of libRadtran.

Radtran is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Radtran is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to the Free
Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

*/

%{

#include <getopt.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "uvspec.h"
#include "../src_c/ascii.h"

static float  yytext2float(char* yytext, int which_token, int max_token);
static float* yytext2floats(char* yytext, int nary);
static int    yytext2nfloats(char* yytext);
static double yytext2double(char* yytext, int which_token, int max_token);
static int    yytext2int(char* yytext, int which_token, int max_token);
static int*   yytext2integers(char* yytext, int nary);
static int    yytext2nintegers(char* yytext);
static char*  yytext2string(char* yytext, int which_token, int max_token);
static int    yytextCstring(char *dest, char* yytext, int which_token, int max_token);
static void free_output(input_struct input, output_struct *output);
static int print_output(input_struct input, output_struct *output);


char* s;
float  tmpfloat;
float* tmp;
int*   tmpi;
int ierror=0;
int start_condition = INITIAL;
int line_number;
input_struct Input;
output_struct Output;

%}

%option noyywrap

DIGIT    [0-9]
SPACE    [ \t]
CCHAR    [#]
COMMENT  ({CCHAR}.*)
INTEGER  {DIGIT}+
SPACE_INTEGER {SPACE}+{INTEGER}
INTEGERS {SPACE_INTEGER}+ 
EXPON   ([DdEe][+-]?{DIGIT}+)
FLOAT    ((({DIGIT}+"."{DIGIT}*)|({DIGIT}+"."{DIGIT}*{EXPON}))|({INTEGER}))
SPACE_FLOAT {SPACE}+{FLOAT}
FLOATS   {SPACE_FLOAT}+ 
SIGN     [-+]
SIGNED_FLOAT {SIGN}?{FLOAT}
SPACE_SIGNED_FLOAT {SPACE}+{SIGNED_FLOAT}
SIGNED_FLOATS {SPACE_SIGNED_FLOAT}+
SIGNED_INTEGER {SIGN}?{INTEGER}
WORD     [_\.\/A-Za-z][_\.\/A-Za-z0-9\-]*

%%

  /* These are the allowed input lines in the uvspec input file 
     The input line specifications must come before the eating of
     blanks, otherwise things get messed up 
     Obviously this whole lot could have been handled more properly
     using bison etc., but than rather few would know what was going 
     on??? 
   */

aerosol_angstrom{SIGNED_FLOATS}     {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.aer.alpha    = yytext2float(s,2,3); 
                                     Input.aer.beta     = yytext2float(yytext,3,3);
				     free(s);}
aerosol_default                     {Input.aer.haze = 1;
                                     Input.aer.vulcan = 1; 
				     Input.aer.seasn = 1;
                                     Input.aer.visib = 50.;}
aerosol_files{SPACE}+{WORD}         {yytextCstring (Input.aer.filename[FN_AER_FILES],
						    yytext, 2, 2);}
aerosol_gg_file{SPACE}+{WORD}       {yytextCstring (Input.aer.filename[FN_AER_GG],
						    yytext, 2, 2);}
aerosol_haze{SPACE}+{INTEGER}       {Input.aer.haze     = yytext2int(yytext,2,2); }
aerosol_moments_file{SPACE}+{WORD}  {yytextCstring (Input.aer.filename[FN_AER_MOM],
						    yytext, 2, 2);}
aerosol_refrac_file{SPACE}+{WORD}   {yytextCstring (Input.aer.filename[FN_AER_REF],
						    yytext, 2, 2);}
aerosol_refrac_index{SPACE_FLOAT}{SPACE_FLOAT} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     tmp   = yytext2floats(yytext, 2); 
                                     Input.aer.re = tmp[0];
				     Input.aer.im = tmp[1];
				     free(tmp); free(s); }
aerosol_scale_ssa{SPACE}+{FLOAT}    {Input.aer.scale_ssa= yytext2float(yytext,2,2); }
aerosol_set_gg{SPACE}+{SIGNED_FLOAT} {Input.aer.set_gg   = yytext2float(yytext,2,2); }
aerosol_set_ssa{SPACE}+{FLOAT}      {Input.aer.set_ssa  = yytext2float(yytext,2,2); }
aerosol_set_tau{SPACE}+{FLOAT}      {Input.aer.set_tau  = yytext2float(yytext,2,2); }
aerosol_season{SPACE}+{INTEGER}     {Input.aer.seasn    = yytext2int(yytext,2,2); }
aerosol_sizedist_file{SPACE}+{WORD} {yytextCstring (Input.aer.filename[FN_AER_SIZ], 
						    yytext, 2, 2);}
aerosol_ssa_file{SPACE}+{WORD}      {yytextCstring (Input.aer.filename[FN_AER_SSA], 
						    yytext, 2, 2);}
aerosol_tau_file{SPACE}+{WORD}      {yytextCstring (Input.aer.filename[FN_AER_TAU],
						    yytext, 2, 2);}
aerosol_visibility{SPACE}+{FLOAT}   {Input.aer.visib    = yytext2float(yytext,2,2); }
aerosol_vulcan{SPACE}+{INTEGER}     {Input.aer.vulcan   = yytext2int(yytext,2,2); }
albedo{SPACE}+{FLOAT}               {Input.atm.albedo   = yytext2float(yytext,2,2); }
albedo_file{SPACE}+{WORD}           {yytextCstring (Input.filename[FN_ALBEDO],
						    yytext, 2, 2);
                                     Input.atm.albedo   = -1.;}
altitude{SPACE}+{SIGNED_FLOAT}      {Input.atm.altitude = yytext2float(yytext,2,2); }
angstrom{SIGNED_FLOATS}             {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.aer.alpha    = yytext2float(s,2,3); 
                                     Input.aer.beta     = yytext2float(yytext,3,3);
				     free(s);}
atmosphere_file{SPACE}+{WORD}       {yytextCstring(Input.filename[FN_ATMOSPHERE],yytext,2,2); }
ch4_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_CH4]   = yytext2float(yytext,2,2); }
co2_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_CO2]   = yytext2float(yytext,2,2); }
correlated_k{SPACE}+{WORD}          {s =  yytext2string(yytext,2,2);
				     if (strncasecmp("kato",s,4)==0)    {Input.ck_scheme = CK_KATO;}
				     else if (strncasecmp("fu",s,2)==0) {Input.ck_scheme = CK_FU;}
				     else if (strncasecmp("avhrr_kratz",s,2)==0) {Input.ck_scheme = CK_AVHRR_KRATZ;}
				     else {fprintf(stderr,"Unknown 'correlated_k %s' on line %d\n", s, line_number+1);
				     ierror++;}
				     free(s);}
crs_file{SPACE}+{INTEGER}{SPACE}+{WORD} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     Input.crs.column   = yytext2int(s,2,3);
                                     Input.crs.filename = yytext2string (yytext, 3, 3); }
data_files_path{SPACE}+{WORD}       {yytextCstring (Input.filename[FN_PATH],
						    yytext, 2, 2); 
                                     /* append slash */
                                     if (Input.filename[FN_PATH][strlen(Input.filename[FN_PATH])-1]!= '/')
				       strcat (Input.filename[FN_PATH], "/");}
day_of_year{SPACE}+{INTEGER}        {Input.day_of_year  = yytext2int(yytext,2,2); }
deltam{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
				     if (strncmp("on",s,2)==0) {Input.rte.deltam = 1;}
				     else if (strncmp(s,"off",3)==0) {Input.rte.deltam = 0;}
				     free(s);}
f11_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_F11]   = yytext2float(yytext,2,2); }
f12_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_F12]   = yytext2float(yytext,2,2); }
f22_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_F22]   = yytext2float(yytext,2,2); }
fisot                               {Input.rte.fisot = 1; }
flexstor                            {Input.flexstor  = 1; }
h2o_continuum{SPACE}+{WORD}         {s = yytext2string(yytext,2,2);
                                     if (strncasecmp("on",s,2)==0) 
				       Input.ck_h2ocont = 1;
				     else {if (strncasecmp(s,"off",3)==0) 
				       Input.ck_h2ocont = 0;}
				     free(s);}
h2o_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_H2O]   = yytext2float(yytext,2,2); }
header                              {Input.header    = 1; }
ic_file{SPACE}+{WORD}               {yytextCstring (Input.filename[FN_ICLOUD],
						    yytext, 2, 2);}
ic_files{SPACE}+{WORD}              {yytextCstring (Input.filename[FN_ICLOUD_FILES], 
						    yytext, 2, 2);}
ic_layer                            {Input.ic.layer=1;}
ic_properties{SPACE}+{WORD}         {s =  yytext2string(yytext,2,2);
                                     if (strncmp("fu",s,2)==0)
				       Input.ic.properties = CLDPROP_DEFAULT;
				     else {
				       if (strncmp("mie",s,3)==0)
					 Input.ic.properties = CLDPROP_MIE;
				       else {
					 Input.ic.properties = CLDPROP_FILE;
					 strcpy (Input.filename[FN_ICLOUD_PROP], s);
				       }
				     }}
ic_scale_gg{SPACE}+{FLOAT}          {Input.ic.scale_gg   = yytext2float(yytext,2,2); }
ic_scale_ssa{SPACE}+{FLOAT}         {Input.ic.scale_ssa  = yytext2float(yytext,2,2); }
ic_set_gg{SPACE}+{FLOAT}            {Input.ic.gg     = yytext2float(yytext,2,2); }
ic_set_ssa{SPACE}+{FLOAT}           {Input.ic.ssa    = yytext2float(yytext,2,2); }
ic_set_tau{SPACE}+{FLOAT}           {Input.ic.tau    = yytext2float(yytext,2,2); }
mc_absorption                       {Input.rte.mc.absorption = MCABS_ABSORPTION; }
mc_basename{SPACE}+{WORD}           {yytextCstring (Input.rte.mc.filename[FN_MC_BASENAME], 
                                                    yytext, 2, 2);}
mc_cloud_grid{SPACE_INTEGER}{SPACE_INTEGER}{SPACE_INTEGER} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     tmpi   = yytext2integers(yytext, 3); 
                                     Input.rte.mc.Nx_cloud = tmpi[0];
				     Input.rte.mc.Ny_cloud = tmpi[1];
				     Input.rte.mc.Nz_cloud = tmpi[2];
				     free(tmpi); free(s); }
mc_emission                         {Input.rte.mc.absorption = MCABS_EMISSION; }
mc_escape                           {Input.rte.mc.escape = 1; }
mc_heating                          {Input.rte.mc.absorption = MCABS_HEATING; }
mc_ipa                              {Input.rte.mc.ipa = 1; }
mc_momcld_file{SPACE}+{WORD}        {s =  yytext2string(yytext,2,2);
                                     if (strncmp("default",s,7)==0)
				       Input.rte.mc.momcld = MOMCLD_DEFAULT;
				     else {
				       if (strncmp("water",s,5)==0)
					 Input.rte.mc.momcld = MOMCLD_WATER;
				       else {
					 if (strncmp("ice",s,3)==0)
					   Input.rte.mc.momcld = MOMCLD_ICE;
					 else {
					   Input.rte.mc.momcld = MOMCLD_FILE;
					   strcpy (Input.rte.mc.filename[FN_MC_MOMCLD], s);
					 }
				       }
				     }}
mc_pcld_file{SPACE}+{WORD}          {s =  yytext2string(yytext,2,2);
				       strcpy (Input.rte.mc.filename[FN_MC_PCLD], s);
				     }
mc_photons{SPACE}+{INTEGER}         {Input.rte.mc.photons  = yytext2int(yytext,2,2); }
mc_photons_file{SPACE}+{WORD}       {yytextCstring(Input.rte.mc.filename[FN_MC_PHOTONS],yytext,2,2); }
mc_sample_grid{SPACE_INTEGER}{SPACE_INTEGER} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     tmpi   = yytext2integers(yytext, 2); 
                                     Input.rte.mc.Nx_sample = tmpi[0];
				     Input.rte.mc.Ny_sample = tmpi[1];
				     free(tmpi); free(s); }
molecular_tau_file{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_MOLABS],yytext,2,2);
                                     Input.molabs = MOLABS_FILE;}
n2o_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_N2O]   = yytext2float(yytext,2,2); }
no_absorption                       {Input.absorption=0; Input.molabs=MOLABS_NONE;}
no_molecular_absorption             {Input.molabs=MOLABS_NONE;}
no_rayleigh                         {Input.rayleigh=RAYLEIGH_NONE;}
nstr{SPACE}+{INTEGER}               {Input.rte.nstr  = yytext2int(yytext,2,2);}
nscat{SPACE}+{INTEGER}              {Input.rte.nscat  = yytext2int(yytext,2,2); }
o3_crs{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
				     if (strncmp("Bass_and_Paur",s,13)==0) {Input.crs.o3_type = O3_BASSPAUR;}
				     else if (strncmp(s,"Molina",6)==0)    {Input.crs.o3_type = O3_MOLINA;}
				     else if (strncmp(s,"Daumont",7)==0)   {Input.crs.o3_type = O3_DAUMONT;}
				     else {fprintf(stderr,"Unknown 'o3_crs %s' on line %d\n", s, line_number+1);
				     ierror++;}
				     free(s);}
ozone_column{SPACE}+{SIGNED_FLOAT}  {Input.ozone_column = yytext2float(yytext,2,2); }
phi0{SPACE}+{SIGNED_FLOAT}          {Input.atm.phi0     = yytext2float(yytext,2,2); }
polradtran_aziorder{SPACE}+{INTEGER} {Input.rte.pol_aziorder  = yytext2int(yytext,2,2); }
polradtran_max_delta_tau{SPACE}+{SIGNED_FLOAT} {Input.rte.pol_max_delta_tau = (double) yytext2float(yytext,2,2); }
polradtran_nstokes{SPACE}+{INTEGER} {Input.rte.pol_nstokes  = yytext2int(yytext,2,2); }
polradtran_quad_type{SPACE}+{WORD}  {Input.rte.pol_quad_type
				       = yytext2string (yytext, 2, 2);}
polradtran_src_code{SPACE}+{INTEGER} {Input.rte.pol_src_code = yytext2int(yytext,2,2); }
pressure{SPACE}+{SIGNED_FLOAT}      {Input.pressure = yytext2float(yytext,2,2); }
prndis{INTEGERS}                    {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.rte.nprndis  = yytext2nintegers(s); 
				     Input.rte.prndis   = 
				       yytext2integers(yytext, Input.rte.nprndis); 
				     free(s); }
rayleigh_tau_file{SPACE}+{WORD}     {yytextCstring(Input.filename[FN_RAYLEIGH],yytext,2,2);
                                     Input.rayleigh = RAYLEIGH_FILE;}
reverse                             {Input.rte.reverse    = 1; }
rpv_file{SPACE}+{WORD}              {yytextCstring(Input.filename[FN_RPV],yytext,2,2);}
rpv_k{SPACE}+{SIGNED_FLOAT}         {Input.rpv.k     = yytext2float(yytext,2,2); }
rpv_rho0{SPACE}+{SIGNED_FLOAT}      {Input.rpv.rho0  = yytext2float(yytext,2,2); }
rpv_theta{SPACE}+{SIGNED_FLOAT}     {Input.rpv.theta = yytext2float(yytext,2,2); }
rte_solver{SPACE}+{WORD}            {s =  yytext2string(yytext,2,2);
				     if (strncmp("sdisort",s,7)==0) {Input.rte.solver = SDISORT;}
				     else if (strncmp(s,"disort2",7)==0) {Input.rte.solver = DISORT2;}
				     else if (strncmp(s,"disort",6)==0) {Input.rte.solver = DISORT;}
				     else if (strncmp(s,"sos",3)==0) {Input.rte.solver = SUCORD;}
				     else if (strncmp(s,"polradtran",6)==0) {Input.rte.solver = POLRADTRAN;}
				     else if (strncmp(s,"twostr",6)==0) {Input.rte.solver = TWOSTR;}
				     else if (strncmp(s,"montecarlo",10)==0) {Input.rte.solver = MONTECARLO;}
				     else {fprintf(stderr,"Unknown 'rte_solver %s' on line %d\n", s, line_number+1);
				     ierror++;}
				     free(s);}
slit_function_file{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_SLITFUNCTION],yytext,2,2);
                                     Input.convolve = 1; }
solar_file{SPACE}+{WORD}            {yytextCstring(Input.filename[FN_EXTRATERRESTRIAL],yytext,2,2); }
source{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
                                     Input.source=SRC_NONE; 
                                      if (strcasecmp("solar",s)==0) 
					Input.source = SRC_SOLAR;
				      else if (strcasecmp(s,"thermal")==0) 
					Input.source = SRC_THERMAL;}
spline{SIGNED_FLOATS}               {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.spline_lambda_0    = yytext2float(s,2,4); 
				     strcpy (s, yytext);
				     Input.spline_lambda_1    = yytext2float(s,3,4); 
                                     Input.spline_lambda_step = yytext2float(yytext,4,4);
                                     Input.spline = 1;
				     free(s);}
spline_file{SPACE}+{WORD}           {yytextCstring(Input.filename[FN_SPLINE],yytext,2,2);
                                     Input.spline = 1; }
surface_temperature{SPACE}+{SIGNED_FLOAT}  {Input.temperature = yytext2float(yytext,2,2); }
sza{SPACE}+{SIGNED_FLOAT}           {Input.atm.sza      = yytext2float(yytext,2,2); }
sza_file{SPACE}+{WORD}              {yytextCstring(Input.filename[FN_SZA],yytext,2,2);
                                     Input.atm.sza      = -1000.0;}

thermal_bandwidth{SPACE}+{FLOAT}    {Input.bandwidth = yytext2double(yytext,2,2); }
transmittance                       {Input.transmittance = 1;}
transmittance_wl_file{SPACE}+{WORD} {yytextCstring(Input.filename[FN_WLTRANS],yytext,2,2); }
umu{SIGNED_FLOATS}                  {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.rte.numu     = yytext2nfloats(s); 
				     Input.rte.maxumu   = Input.rte.numu;
				     Input.rte.umu      = 
				       yytext2floats(yytext, Input.rte.numu); 
				     free(s); }
phi{SIGNED_FLOATS}                  {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.rte.nphi     = yytext2nfloats(s); 
				     Input.rte.maxphi   = Input.rte.nphi;
				     Input.rte.phi      = 
				       yytext2floats(yytext, Input.rte.nphi); 
				     free(s); }
wc_cloudcover{SPACE}+{FLOAT}        {Input.cloudcover= yytext2float(yytext,2,2); 
                                     Input.ipa=1;}
wc_file{SPACE}+{WORD}               {yytextCstring(Input.filename[FN_WCLOUD],yytext,2,2); }
wc_files{SPACE}+{WORD}              {yytextCstring (Input.filename[FN_WCLOUD_FILES],
						    yytext, 2, 2);}
wc_ipa                              {Input.ipa=1;}
wc_layer                            {Input.wc.layer=1;}
wc_properties{SPACE}+{WORD}         {s =  yytext2string(yytext,2,2);
                                     if (strncmp("hu",s,2)==0)
				       Input.wc.properties = CLDPROP_DEFAULT;
				     else {
				       if (strncmp("mie",s,3)==0)
					 Input.wc.properties = CLDPROP_MIE;
				       else {
					 Input.wc.properties = CLDPROP_FILE;
					 strcpy (Input.filename[FN_WCLOUD_PROP], s);
				       }
				     }}
wc_scale_gg{SPACE}+{FLOAT}          {Input.wc.scale_gg   = yytext2float(yytext,2,2); }
wc_scale_ssa{SPACE}+{FLOAT}         {Input.wc.scale_ssa  = yytext2float(yytext,2,2); }
wc_set_gg{SPACE}+{FLOAT}            {Input.wc.gg   = yytext2float(yytext,2,2); }
wc_set_ssa{SPACE}+{FLOAT}           {Input.wc.ssa  = yytext2float(yytext,2,2); }
wc_set_tau{SPACE}+{FLOAT}           {Input.wc.tau    = yytext2float(yytext,2,2); }
wvn{FLOATS}                         {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
				     Input.wl.wl_start  = yytext2float(s,2,3); 
                                     Input.wl.wl_end    = yytext2float(yytext,3,3);
				     if (Input.wl.wl_start > Input.wl.wl_end) {
				       tmpfloat = Input.wl.wl_start;
				       Input.wl.wl_start = Input.wl.wl_end;
				       Input.wl.wl_end = tmpfloat;
				     }
				     free(s);}
zout{SIGNED_FLOATS}                 {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.atm.nzout    = yytext2nfloats(s); 
                                     Input.atm.zout     = 
				       yytext2floats(yytext, Input.atm.nzout); 
				     free(s); }



{SPACE}+       ;         /* eat up blanks */ 

{COMMENT}*     ;         /* eat up comments */

\n                        { ++line_number;};

{WORD}                    {fprintf(stderr,"Unknown command on line %d\n", line_number+1);
                           fprintf(stderr,"%s\n", yytext);
                           ierror++;}

<<EOF>> { 
  if ( ierror > 0 ) { exit(1); }
	    yyterminate();
        }      

%%

/*************************************************************/
/* print usage information                                   */
/*************************************************************/

static void usage()
{

  fprintf (stderr, "\nuvspec, version %4.2f\n", VERSION);
  fprintf (stderr, "\nCalculates the 1D radiation field of the Earth\'s atmosphere.\n");
  fprintf (stderr, "See libRadtran documentation for more information.\n\n");
  fprintf (stderr, "No error checking yet, so be careful.\n\n");

  fprintf (stderr, "Usage: uvspec [-h] < input_file > Output_file\n");
  fprintf (stderr, " -h    Print this message.\n");  
  fprintf (stderr, "\n");  
}

int main(int argc, char **argv) 
{
  
  /*  @50c@                

@code{uvspec} calculates the radiation field in the Earth's atmosphere
for a given set of input parameters. It reads input from standard input, 
and outputs to standard output. It is normally invoked in the following
way: 

@example
uvspec < input_file > output_file
@end example

@noindent The format of the input and output files are described
below. Several realistic examples of input files are subsequently 
given.

@strong{Warning:} Please note the error checking on input variables 
is very scarce at the moment. Hence, if you provide erroneous input, 
the outcome is unpredictable.
  @c50@ */

  /*  @50_01c@                
@cindex input file, uvspec
@cindex uvspec, input file

The uvspec input file consists of single line entries, each making
up a complete input to the uvspec program. First on the line comes
the parameter name, followed by one or more parameter values. The 
parameter name and the parameter values are separated by white space.
Filenames are entered without any surrounding single or double quotes.
Comments are introduced by a @code{#}. Blank lines are ignored. 

The various input parameters are described in detail below.
@vtable @code
@item aerosol_angstrom
Scale the aerosol optical depth using the @AA{}ngstr@"om formula. Specify 
the @AA{}ngstr@"om alpha and beta coefficients. @code{aerosol_vulcan}, 
@code{aerosol_season}, @code{aerosol_haze} and @code{aerosol_visibility} must also
be specified.
@item aerosol_default
Set up a default aerosol according to 
E.P. Shettle, "Models of aerosols, clouds and precipitation for 
atmospheric propagation studies", in "Atmospheric propagation in the 
uv, visible, ir and mm-region and related system aspects", 
AGARD Conference Proceedings (454), 1989. The default properties are 
a rural type aerosol in the boundary layer, background aerosol above 2km,
spring-summer conditions and a visibility of 50km. These settings may
be modified with @code{aerosol_haze}, @code{aerosol_vulcan}, @code{aerosol_season},
and @code{aerosol_visibility}.
@item aerosol_files
A way to specify aerosol optical depth, single scattering albedo, 
and phase function moments for each layer. The file specified by 
@code{aerosol_files} has two columns where column 1 is the altitude in km. The altitudes 
must be the same as those specified in the file @code{atmosphere_file}. The second
column is a the name of a file which defines the optical properties of the level 
starting at the given altitude. The files specified in the second column must have the 
following format:
@table @code
@item Column 1: 
The wavelength in nm. These wavelengths may be different from those in @code{solar_file}. 
Optical properties are interpolated to the requested wavelengths.
@item Column 2:
The extinction coefficient of the layer in units km-1. 
@item Column 3:
The aerosol single scattering albedo of the layer.
@item Column 4-(nmom+4):
The moments of the aerosol phase function. 
@end table
For some simple examples see the files @file{examples/AERO_*.LAYER}. Note that 
if using the @code{rte_solver disort2} it makes good sense to make the 
number of moments larger than @code{nstr}. For @code{rte_solver disort} and 
@code{rte_solver polradtran} the number of moments included in the calculations 
will be @code{nstr}+1. Higher order moments will be ignored for these solvers.
@item aerosol_gg_file
Location of aerosol asymmetry parameter file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the asymmetry parameter of each layer. 
At present, the asymmetry parameter defined here is constant with wavelength
but it is planned to introduce the wavelength dependence in the near future.
Comments start with @code{#}. Empty lines are ignored.
@item aerosol_haze
Aerosol type in the lower 2 km of the atmosphere. Integer.
See E.P. Shettle, "Models of aerosols, clouds and precipitation for 
atmospheric propagation studies", in "Atmospheric propagation in the 
uv, visible, ir and mm-region and related system aspects", 
AGARD Conference Proceedings (454), 1989.

@table @code
@item 1
Rural type aerosols.
@item 4
Maritime type aerosols.
@item 5
Urban type aerosols.
@item 6
Tropospheric type aerosols.
@end table
@item aerosol_moments_file
Location of aerosol moments file, a one-column file containing an
arbitrary number of Legendre terms of the phase function. The phase function 
@iftex
 @tex
 $p(\mu)$ is @*
  $$ p (\mu) = \sum_{m=0}^{\infty} (2m+1) \cdot k_m \cdot P_m (\mu) $$
  where $k_m$ is the m'th moment and $P_m (\mu)$ is the m'th Legendre polynomial.
 @end tex
@end iftex
@ifinfo
 p(mu) is @*
 p (mu) = sum (m=0 to infinity) (2m+1) * k(m) * Pm (mu) @*
 where k(m) is the m'th moment and Pm (mu) is the m'th Legendre polynomial.
@end ifinfo
If not specified, a Henyey-Greenstein phase function is assumed 
where the asymmetry parameter g is either a default value depending on the 
aerosol type or may be specified using @code{aerosol_set_gg}.
@item aerosol_refrac_file
File containing the wavelength-dependent refractive index of the aerosol;
together with @code{aerosol_sizedist_file} this forms the 
input to Mie calculations of the aerosol optical properties. Attention: 
If the aerosol properties are defined using the refractive index and the 
size distribution, the wavelength dependence of the optical properties is 
determined by Mie theory. At present there are three ways to define the 
absolute value of the optical thickness: (1) @code{visibility} defines the 
profile at the first @emph{internal} wavelength; for a monochromatic calculation
and in correlated-k mode, the first @emph{internal} wavelength equals the 
first wavelength output by @code{uvspec}; for spectral calculations, the 
first wavelength might be a little bit smaller than the first wavelength 
output by @code{uvspec}; (2) @code{aerosol_tau_file} defines the optical 
thickness profile at the first @emph{internal} wavelength; or (3) absolute
optical thickness and wavelength-dependence are defined by @code{aerosol_angstrom}.
In future it is planned to also allow the specification of 
absolute particle densities, etc.
@item aerosol_refrac_index
Wavelength-independent refractive index of the aerosol; if wavelength-dependence 
is required, use @code{aerosol_refrac_file} instead. Together with 
@code{aerosol_sizedist_file} this forms the input to Mie calculations of the 
aerosol optical properties. See comment at @code{aerosol_refrac_file}.
@item aerosol_scale_ssa
Scale the aerosol single scattering albedo for all wavelengths and altitudes
with a positive number. If the resulting scaled single scattering
albedo is larger than 1 it is set to 1.
@item aerosol_set_gg
Set the aerosol asymmetry parameter for all wavelengths and altitudes
to a constant value between -1.0 and 1.0.
@item aerosol_set_ssa
Set the aerosol single scattering albedo for all wavelengths and altitudes
to a constant value between 0.0 and 1.0.
@item aerosol_set_tau
Set the aerosol optical thickness for all wavelengths and altitudes
to a constant value
@item aerosol_season
Specify season to get appropriate aerosol profile.
@table @code
@item 1
Spring-summer profile.
@item 2
Fall-winter profile.
@end table
@item aerosol_sizedist_file
Aerosol size distribution. Together with @code{aerosol_refrac_index} or
@code{aerosol_refrac_file} this forms the input to Mie calculations of the 
aerosol optical properties. See comment at @code{aerosol_refrac_file}.
@item aerosol_ssa_file
Location of aerosol single scattering albedo file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the single scattering albedo of each layer. 
At present, the single scattering albedo defined here is constant with wavelength
but it is planned to introduce the wavelength dependence in the near future.
Comments start with @code{#}. Empty lines are ignored.
@item aerosol_tau_file
Location of aerosol optical depth file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the aerosol optical depth of each layer. 
To get wavelength dependence use the parameter @code{aerosol_angstrom}. 
Comments start with @code{#}. Empty lines are ignored.
@item aerosol_visibility
Visibility in km.
@item aerosol_vulcan
Aerosol situation above 2 km. Integer.
@table @code
@item 1
Background aerosols.
@item 2
Moderate vulcanic aerosols.
@item 3
High vulcanic aerosols.
@item 4
Extreme vulcanic aerosols.
@end table
@item albedo
The Lambertian surface albedo, a number between 0.0 and 1.0,
constant for all wavelengths. For wavelength dependent surface albedo 
use @code{albedo_file}.
@item albedo_file
Location of surface albedo file for wavelength dependent surface albedo. 
The file must have two columns.
Column 1 is the wavelength, in nm, and column 2 the corresponding 
Lambertian surface albedo. The wavelength grid may be freely set. The 
albedo will be interpolated to the wavelength grid used for the radiation
calculation. Comments start with @code{#}. Empty lines are ignored.
@item angstrom
Still supported but obsolete. Replaced by @code{aerosol_angstrom}.
@item altitude
Set the bottom level in the model atmosphere provided in 
@code{atmosphere_file} to be at altitude (km).
@example
altitude 0.73   # Altitude of IFU, Garmisch-Partenkirchen 
                # Be aware, for this to work the atmosphere
                # file must start at 0 km.
@end example
The profiles of pressure, temperature, molecular absorbers, 
ice and water clouds are cut at the specified altitude. 
The aerosol profile is not affected by @code{altitude} but starts
right from the model surface. This is a convenient way for the user to calculate the 
radiation at other altitudes than sealevel. Note that @code{altitude} is very different
from @code{zout} where the radiation is calculated at an altitude of zout
over the surface. E.g. to calculate the radiation field 1 km above the surface 
at 0.73 above sealevel, one would specify '@code{altitude} 0.73' 
and '@code{zout} 1.0'.
@item atmosphere_file
Location of the atmospheric data file. The nine columns of the file 
are interpreted as follows:
@table @code
@item 1. z(km)
Altitude above sea level in km
@item 2. p(mb)
Pressure in mbar
@item 3. T(K)
Temperature in K
@item 4. air_density(cm-3) 
Air density in cm-3
@item 5. o3_density(cm-3)
Ozone density in cm-3
@item 6. o2(cm-3)
Oxygen density in cm-3
@item 7. h2o(cm-3) 
Water vapour density in cm-3
@item 8. co2(cm-3)
CO2 density in cm-3
@item 9. no2(cm-3)
NO2 density in cm-3
@end table
At minimum, the first five columns must be specified. Columns 6 - 9
may be omitted. Missing profiles are filled with zero's. The atmosphere is specified
top-down, that is, the top level is the first line in the file, the bottom
(surface) level the last line. All properties refer to model @emph{level} z,
not to model @emph{layer}. It is important that the correct units are 
used, otherwise unpredictable results are guaranteed.
Comments start with @code{#}. Empty lines are ignored.
@item ch4_mixing_ratio
The mixing ratio of CH4 in ppm
(default: 1.6 ppm).
@item co2_mixing_ratio
The mixing ratio of CO2 in ppm
(default: calculated from the lowest level in the atmospheric data file).
@item correlated_k
To calculate integrated shortwave or longwave irradiance, or to simulate 
satellite instrument channels, choose between the following 
correlated-k schemes:
@table @code
@item Kato 
Kato et al. [1999], shortwave
@item Fu
Fu and Liou [1992/93], shortwave and longwave
@item AVHRR_KRATZ
Kratz [1995], AVHRR instrument channels
@end table
If correlated_k is specified, the extraterrestrial flux is taken from
internally defined files specific for each parameterization, not 
from @code{solar_file}. The output is the integrated irradiance for 
each band. To get e.g. integrated shortwave irradiance, simply add all 
bands of the Kato et al. or Fu and Liou parameterization. 
The five AVHRR channels are weighted sums of the libRadtran output. 
Examples how to integrate the output in the @code{AVHRR_KRATZ} case 
are included in the @code{uvspec} self-test which is initiated with 
@code{make check}.
@item data_files_path
The path to the directory where all uvspec internal data files live, e.g.
the files that are in the subdirectories of the @code{data} directory
that comes with the uvspec distribution.
@item day_of_year
Integer, to correct the calculated radiation quantities for the 
Sun-Earth distance for the specified Julian day (1-365). If not given, 
the Earth-Sun distance is 1 AU (i.e. equinox distances), 
that is, no correction is applied to the extraterrestrial 
irradiance @code{solar_file}.
@item deltam
Turn delta-M scaling on/off. Set to either @code{on} or @code{off}. Note 
that for the @code{rte_solver disort2} delta-M scaling is hardcoded to 
always be on.
@item f11_mixing_ratio
The mixing ratio of F11 in ppm (default: 0.000268 ppm).
@item f12_mixing_ratio
The mixing ratio of F12 in ppm (default: 0.000503 ppm).
@item f22_mixing_ratio
The mixing ratio of F22 in ppm (default: 0.000105 ppm).
@item fisot
Specifies that isotropic illumination is used at top-boundary instead
of beam source. Useful for those who want to calculate the reflectance 
for a homogeneous or inhomogeneous atmosphere. The intensity is still set by
@code{solar_file}.
@item flexstor
Output is in flexstor format. May not be combined with @code{header}.
Also, does not currently work when @code{umu} and/or @code{phi} is specified.
@item h2o_mixing_ratio
The mixing ratio of H2O in ppm. The H2O profile is scaled by a factor which
is determined from the ratio of the specified mixing ratio and the value
in the atmospheric data file in the lowermost layer.
@item header
Output information on some of the input parameters. May not be combined
with @code{flexstor}.
@item n2o_mixing_ratio
The mixing ratio of N2O in ppm
(default: 0.28 ppm).
@item ic_file
Location of file specifying optional ice cloud properties. The file format 
is indentical to @code{wc_file}. Note the following: The third column is interpreted 
as effective radius with 'ic_properties mie' and as 'generalized effective size'
with 'ic_properties fu' (the default), see equation (2.3) in 
Fu (An accurate parameterization of the solar radiative properties of cirrus 
clouds for climate models, J. Clim., 9, 2058-2082, 1996) or equation (2.2b) in
Fu et al. (An accurate parameterization of the infrared radiative properties 
of cirrus clouds for climate models, J. Clim., 11, 2223-2237, 1998). The 
'generalized effective diameter' is something like an effective size for 
hexagonal prisms.
@item ic_files
A way to specify ice cloud optical depth, single scattering albedo, 
and phase function moments for each layer. The file specified by 
@code{ic_files} has two columns where column 1 is the altitude in km. The altitudes 
must be the same as those specified in the file @code{atmosphere_file}. The second
column is the name of a file which defines the optical properties of the level 
starting at the given altitude. The files specified in the second column must 
have the following format:
@table @code
@item Column 1: 
The wavelength in nm. These wavelengths may be different from those in @code{solar_file}. 
Optical properties are interpolated to the requested wavelengths.
@item Column 2:
The extinction coefficient of the layer in units km-1. 
@item Column 3:
The single scattering albedo of the layer.
@item Column 4-(nmom+4):
The moments of the scattering phase function. 
@end table
Note that if using the @code{rte_solver disort2} it makes good sense to make the 
number of moments larger than @code{nstr}. For @code{rte_solver disort} and 
@code{rte_solver polradtran} the number of moments included in the calculations 
will be @code{nstr}+1. Higher order moments will be ignored for these solvers.
@item ic_layer
Specify ice cloud properties for model @emph{layers} instead of @emph{levels} 
(see also @code{ic_file}). 
@item ic_properties
Defines how ice water content and effective particle radius are translated 
to optical properties. Possible choices are
@table @code
@item fu 
Parameterization by Fu [1996] and Fu et al. [1998] (see ic_file); works only
with @code{correlated_k FU}; this is the default setting.
@item mie
Use pre-calculated Mie tables; the tables are expected in 
@code{data_files_path}/correlated_k/.../
@item filename
Read optical properties from specified filename; file format is as produced 
by Frank Evans' cloudprp; for each of the internal (computational) wavelengths, 
a separate file is expected (THIS OPTION IS SUBJECT TO CHANGE).
@end table
@item ic_scale_gg
Scale the ice cloud asymmetry factor for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item ic_scale_ssa
Scale the ice cloud single scattering albedo for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item ic_set_gg
Set the ice cloud asymmetry factor for all wavelengths and altitudes
to a float between -1.0 and 1.0.
@item ic_set_ssa
Set the ice cloud single scattering albedo for all wavelengths
and altitudes to a value between 0.0 and 1.0.
@item ic_set_tau
Set the total ice cloud optical depth to a constant value for all wavelengths.
This option is useful only for monochromatic calculations or in wavelength regions 
where the optical properties of ice clouds can be considered constant, e.g. the 
ultraviolet region.
@item mc_absorption
Calculate MYSTIC absorption of each grid box; may need a considerable amount of memory,
depending on the 3D cloud grid. Only meaningful with @code{rte_solver montecarlo}.
@item mc_basename
Filename for MYSTIC 3D output.
@item mc_cloud_grid
3D cloud grid size (Nx,Ny,Nz) for MYSTIC. Weird, but required to keep MYSTIC 
a function that can be called from a Fortran program.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_emission
Calculate MYSTIC emission of each grid box; may need a considerable amount of memory,
depending on the 3D cloud grid. Note that emission is calculated directly without tracing 
any photons. Increasing @code{mc_photons} will therefore not change the result.
Only meaningful with @code{rte_solver montecarlo} and @code{source thermal}. 
@item mc_escape
Calculate MYSTIC radiances via escape probabilities; slows down the tracing 
but usually requires much less photons. Only meaningful with  @code{rte_solver montecarlo}.
@item mc_heating
Calculate MYSTIC heating rates for each grid box; may need a considerable amount of memory,
depending on the 3D cloud grid. Note that emission is calculated directly without tracing 
any photons. Increasing @code{mc_photons} will therefore not change the result.
Only meaningful with @code{rte_solver montecarlo} and @code{source thermal}. 
@item mc_ipa
Run MYSTIC in independent pixel mode. Only meaningful with @code{rte_solver montecarlo}.
@item mc_momcld_file
Cloud phase function moments file for MYSTIC; can be any filename or 
'water' or 'ice'. In the latter case, pre-calculated tables for water 
and ice are used. 
Only meaningful with @code{rte_solver montecarlo}.
@item mc_photons
Total number of photons to be traced by the Monte Carlo solver, MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_pcld_file
Cloud phase function file for MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_photons
Total number of photons to be traced by the Monte Carlo solver, MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_photons_file
Distribution of photons over wavelength bands; to be used with @code{correlated_k};
for an example see @file{data/correlated_k/kato/photons.dat}. No error checking yet!
Do only use if you are absolutely sure what you are doing.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_sample_grid
Sampling grid size (Nx,Ny) for MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item molecular_tau_file
Location of molecular absorption optical depth file. Usually, molecular absorption
is calculated from trace gas concentrations provided in @code{atmosphere_file}
(scaled with @code{ozone_column}, etc. Use this option only if you really want to specify
the optical depth directly (e.g. for a model intercomparison). Note also that 
no wavelength dependence can be specified; hence this option is only useful for 
monochromatic calculations. The  @code{absorption_tau_file} must have two columns. 
Column 1 is the altitude in km where the altitude grid must be exactly equal 
to the altitude grid specified in @code{atmosphere_file}. 
Column 2 is the absorption optical depth of each layer.
Comments start with @code{#}. Empty lines are ignored.
@item no_absorption
Switch absorption off.
@item no_molecular_absorption
Switch all (molecular, aerosol, cloud, and ice cloud) absorption off. Note that
this option does not affect the single scattering albedo; e.g. with an aerosol
optical depth of 1 and a single scattering albedo of 0.7, the scattering optical 
depth will still be 0.7, even with @code{no_absorption}. 
@item no_rayleigh
Switch Rayleigh scattering off.
@item nscat
The order of scattering for the @code{sos} radiative transfer equation solver. 
Default is 20.
@item nstr
Number of streams used to solve the radiative transfer equation.
Default is 6. If changed, make sure that @code{mxcmu} in 
@file{src_f/disort.f} and @file{src_f/sdisort.f} are larger or equal 
to @code{nstr}. If not, change @code{mxcmu} and recompile.
@item o3_crs
Choose between the following ozone cross sections.
@table @code
@item Bass_and_Paur
Bass and Paur ozone cross section.
@item Molina
Molina and Molina ozone cross section.
@item Daumont
Daumont et al. ozone cross section.
@end table
Molina and Molina is default.
@item ozone_column
Set the ozone column to a given value. The value must be in 
Dobson units. If value is negative or @code{ozone_column}
is not specified, the ozone column is not scaled.
@item phi
Azimuth output angles (in degrees) in increasing order. 
The radiance is output at @code{phi} and @code{umu}.
@item phi0
Azimuth angles of the sun (0 to 360 degrees). If @code{phi0} varies as a
function of wavelength use @code{sza_file}.
@item polradtran_aziorder
Order of Fourier azimuth series: 0 is azimuthially symmetric case.
Default 0.
@item polradtran_max_delta_tau
Initial layer thickness for doubling; governs accuracy, 10E-5 should be
adequate. Do not go beyond half the real precision, i.e. 10e-8 for REAL*8.
Default 1.e-05.
@item polradtran_nstokes
Number of Stokes parameters
@table @code
@item 1 
for I (no polarization)
@item 2
for I,Q
@item 3
for I,Q,U
@item 4 
for I,Q,U,V
@end table
Default is 1.
@item polradtran_quad_type
Type of quadrature used: 
@table @code
@item G
gaussian
@item D
double gaussian, 
@item L
Lobatto
@item E
extra-angle
@end table
Default G.                                  
@item polradtran_src_code
Radiation sources included:
@table @code
@item 0
none
@item 1
solar
@item 2
thermal
@item 3
both
@end table
Default 1.
@item pressure
The surface pressure in mb. The pressure profile and density profiles are 
scaled accordingly.
@item prndis
Specify one or more integers between 1 and 7.
Print various disort input and output in disorts own format. See 
@file{disort.doc} for more information.
@strong{Warning:} Produces a lot of output. 
@item rayleigh_tau_file
Location of Rayleigh scattering optical depth file. Usually, the Rayleigh scattering
cross section is calculated from the air pressure provided in @code{atmosphere_file}
(scaled with @code{pressure}. Use this option only if you really want to specify
the optical depth directly (e.g. for a model intercomparison). Note also that 
no wavelength dependence can be specified; hence this option is only useful for 
monochromatic calculations. The  @code{rayleigh_tau_file} must have two columns. 
Column 1 is the altitude in km where the altitude grid must be exactly equal 
to the altitude grid specified in @code{atmosphere_file}. 
Column 2 is the Rayleigh scattering optical depth of each layer.
Comments start with @code{#}. Empty lines are ignored.
@item reverse
Option for the strong and bold. Reverses the atmospheric input to the radiative
transfer solvers. That is, the atmosphere is turned on the head.
Yes, that is actually useful for some purposes. If you think
you need this contact the author. Otherwise, do not use.
@item rpv_file
4 column file, containing the Rahman, Pinty, and Verstraete (RPV) BDRF parameterization 
Rahman, Pinty, and Verstraete, "Coupled Surface-Atmosphere Reflectance 
(CSAR) Model. 2. Semiempirical Surface Model Usable With NOAA Advanced 
Very High Resolution Radiometer Data, J. Geophys. Res., 98, 20791-20801, 1993).
Bidirectional reflectance distribution functions for a variety of surfaces are 
given in the paper. At present, this option is only used with the MYSTIC solver, 
probably later also with DISORT 2.0. 
The columns of the input file are wavelength [nm], rho0, k, and theta. 
The parameters are interpolated linearely to the internal wavelength grid. 
This procedure is not tested yet.
To make sure that the results are reasonable, specify the RPV data on a wavelength 
grid similar or equal to that used internally for the radiative transfer calculation!
@item rpv_k
Constant RPV rho0, see @code{rpv_file}. @code{rpv_k} overwrites the wavelength-dependent
value defined in @code{rpv_file}.
@item rpv_rho0 
Constant RPV rho0, see @code{rpv_file}. @code{rpv_rho0} overwrites the wavelength-dependent
value defined in @code{rpv_file}.
@item rpv_theta
Constant RPV theta, see @code{rpv_file}. @code{rpv_theta} overwrites the wavelength-dependent
value defined in @code{rpv_file}.
@item rte_solver
Set the radiative transfer equation solver to be used. Options are
@vtable @code
@item disort
The standard plane--parallel disort algorithm by Stamnes et al. (1988).
@item disort2
Version 2 of disort. See disort documentation for details of changes.
@item sdisort
Pseudospherical disort as described by Dahlback and Stamnes (1991).

@strong{Warning:} it is not recommended to use sdisort for cloudy conditions. 
With clouds included it may be numerically unstable and produce 
wrong results.
@item polradtran
The plane-parallel radiative transfer solver of Evans and Stephens (1991).
Includes polarization. It has no effect to use @code{umu} with this solver. 
However @code{phi} may be used. Note that polarization effects of aerosols and 
clouds are currently not included.
@item twostr
The two--stream radiative transfer solver described by Kylling, Stamnes
and Tsay (1995).
@item sos
A scalar pseudospherical succesive orders of scattering code. Works for 
solar zenith angles smaller than 90 degrees. Can calculate azimuthally 
averaged radiances. Set @code{nscat} to specify the order of scattering.
@end vtable
@item slit_function_file
If specified, the calculated spectrum is convolved with the function found
in the @file{slit_function_file}. The file must
contain two columns. Column 1 is the wavelength, in nm, and relative to the
center wavelength. Column 2 is the corresponding slit function value. It must
be unity at the maximum. The wavelength steps in the slit function file must
be equidistant. Comments start with @code{#}. Empty lines are ignored.
@item solar_file
Location of file holding the extraterrestrial spectrum. The file must
contain two columns. Column 1 is the wavelength, in nm, and column 2 
the corresponding extraterrestrial flux. The user may freely use any units
he/she wants on the extraterrestrial flux. The wavelength grid specified, 
will be the wavelength resolution at which results are returned. However, 
the wavelength range is determined by @code{wvn}. 
Comments start with @code{#}. Empty lines are ignored.
Note that @code{solar_file} is ignored if @code{correlated_k} is specified.
@item source
Solar or thermal source. Set to either @code{solar} or @code{thermal}.
@item spline
Spline interpolate to wavelengths lambda_0 to lambda_1 in steps of lambda_step,
in nm. Specified as e.g.
@example
spline 290. 365. 0.5
@end example
Here, the calculated spectrum is interpolated to wavelengths 290., 290.5, 291., ...
, 364.5, 365. For interpolation to arbitrary wavelengths use @code{spline_file}.
The specified wavelength interval must be within the one specified by @code{wvn}.
@item spline_file
Spline interpolate to arbitrary wavelengths, in nm, given as a single column in file 
@file{spline_file}. The specified wavelengths must be within the range specified 
by @code{wvn}. Comments start with @code{#}. Empty lines are ignored.
@item surface_temperature
Surface temperature, used for thermal infrared calculations. If not specified, the
temperature of the lowest atmospheric level is used as surface temperature.
@item sza
The solar zenith angle. If the solar zenith angle varies with wavelength,
use @code{sza_file}. 
@item sza_file
Location of solar zenith angle file for wavelength dependent solar
zenith angle. The file must have two or three columns. Column 1 is the wavelength, 
in nm, and column 2 the corresponding  solar zenith angle. Optionally
the third column may contain the corresponding solar azimuth angle. The solar 
azimuth angle is only needed when calculating radiances. The wavelength 
grid may be freely set. The solar zenith and azimuth angle will be interpolated to 
the wavelength grid used for the radiation calculation.
Comments start with @code{#}. Empty lines are ignored.
@item thermal_bandwidth
Specify a bandwidth in cm-1 for monochromatic thermal calculations. 
The default is 1 cm-1.
@item transmittance
Calculate transmittance / reflectance. That is, set the extraterrestrial 
irradiance to 1 and do not correct for Sun-Earth distance.
@item transmittance_wl_file
Location of single column file that sets the wavelength grid used for the 
internal transmittance calculations. The wavelengths must be in nm.
Do not use this option unless you know what you are doing.
Comments start with @code{#}. Empty lines are ignored.
@item umu
Cosine of output polar angles in increasing order, starting with 
negative (downward) values (if any) and on through positive (upward)
values. Must not have any zero values. The azimuthally averaged intensity
@code{u0u} is output at @code{umu}.
@item wc_cloudcover
Set the fraction of the horizontal sky area which is covered by clouds.
When a cloud cover is specified, the result will be calculated by
the independent pixel approximation (IPA), that is, as weighted average 
of cloudless sky and overcast sky, where the cloud properties are taken 
from @code{wc_file}, etc. When @code{wc_cloudcover} is specified, 
@code{wc_ipa} is switched on
automatically (see also 
@code{wc_ipa}).
@item wc_file
Location of file defining water cloud properties. The file must
contain three columns. Column 1 is the altitude in km, column 2 
the liquid water content in grams per qubic meter, and column 3 the effective
droplet radius in micron. The liquid water content and effectice droplet 
radius must be specified at the same altitude grid as specified
in the @code{atmosphere_file}. Note that the definition of cloud altitudes   
in @code{wc_file} refers to sea level, not to altitude above
ground. E.g., when altitude is set to 1.63km, and the first cloud level is 
defined at 3km, the cloud would start at 1.37km above ground.
The optical depth of a layer is calculated 
using information from the upper and lower levels defining the layer. 
Comments start with @code{#}. Empty lines are ignored. An example 
of a cloud defined by three levels and thus covering two model layers is given in 
the file @file{examples/WC.DAT}.  
The parameterization of Hu and Stamnes (An accurate parameterization of the 
radiative properties of water clouds suitable for use in climate models, 
J. Clim., 6, 728-742, 1993) is used. Per default, the cloud properties are 
interpreted as properties at a given altitude @emph{level}. If @code{wc_layer} is defined,
they are interpreted as @emph{layer} properties.
@item wc_files
A way to specify cloud extinction coefficient, single scattering albedo, 
and scattering phase function for each layer. The file specified by 
@code{wc_files} has two columns where column 1 is the altitude in km. The altitudes 
must be the same as those specified in the file @code{atmosphere_file}. The second
column is the name of a file which defines the optical properties of the level 
starting at the given altitude. The files specified in the second column must 
have the following format:
@table @code
@item Column 1: 
The wavelength in nm. These wavelengths may be different from those in @code{solar_file}. 
Optical properties are interpolated to the requested wavelengths.
@item Column 2:
The extinction coefficient of the layer in units km-1. 
@item Column 3:
The single scattering albedo of the layer.
@item Column 4-(nmom+4):
The moments of the scattering phase function. 
@end table
Note that if using the @code{rte_solver disort2} it makes good sense to make the 
number of moments larger than @code{nstr}. For @code{rte_solver disort} and 
@code{rte_solver polradtran} the number of moments included in the calculations 
will be @code{nstr}+1. Higher order moments will be ignored for these solvers.
@item wc_ipa
Independent pixel approximation (IPA) of a 3D cloud field. Cloud information
is read from a file called wcloud3D.dat. The three-dimensional cloud 
field is split into individual columns which then are considered 
one-dimensional and treated separately with the specified RTE solver.
The output quantities are averages over all columns. If wc_cloudcover is 
also specified, the cloud is not read from wcloud3D.dat but from wc_file
(see also wc_cloudcover). The format of wcloud3D.dat is as follows:

@example

  80 80 94 1 1
  0.025 0.025 0.0 0.5 0.75 1.0 1.4 1.5 ...
   
  # layer 3,  0.75 - 1.0 km
  1 80 4 1.25 0.86 0.99999
  2 80 4 1.25 0.86 0.99999
  3 80 4 1.25 0.86 0.99999
  4 80 4 1.25 0.86 0.99999
  ...
@end example

@enumerate    
@item 1st line: Nx  Ny  Nz  gspec wspec
 
@table @asis
@item   Nx     
number of boxes in x direction
@item   Ny     
number of boxes in y direction
@item   Nz     
number of boxes in z direction
@item   gspec  
if > 0, asymmetry parameter is specified for each cell
@item   wspec  
if > 0, single scattering albedo is specified for each cell

@end table
Grid boxes are always counted from 1 .. Nx, 1 .. Ny, 1 .. Nz.
@item 2nd line: dx  dy  z[1] z[2] z[3] ... z[Nz] z[Nz+1]
@table @asis
@item  dx  
size of boxes in x direction [km]
@item  dy  
size of boxes in y direction [km]
@item  z[1] .. z[Nz+1]   
z-grid [km]. Note that this grid must be exactly the same 
as the 1D grid in @code{atmosphere_file}
@end table
@item 3rd line, etc.   (cloud definition lines)
@example
       ix  iy  iz  ext  g  w
@end example
@table @asis
@item   ix   
index of grid box in x direction  (1 .. Nx)
@item   iy   
index of grid box in y direction  (1 .. Ny)
@item   iz   
index of grid box in z direction  (1 .. Nz)
@item   ext  
extinction coefficient [1/km]
@item   g    
asymmetry parameter (only used if gspec > 0)
@item   w    
single scattering albedo (only used if wspec > 0)
@end table
@end enumerate
 
@emph{Rules to set up the atmosphere:}

@enumerate
@item An 1D atmosphere is built by uvspec, as usual. This atmosphere
      may contain 1D clouds, defined in @code{wc_filename}.
@item Each layer for which a cloud definition line is found in
      @code{wcloud3D.dat} will be overwritten by 3D clouds. 
      If gspec or wspec are not set,
      asymmetry parameter or single scattering albedo will be
      taken from the 1D definition. 
@item If only properties for some boxes of a layer are defined,
      the extinction coefficients of all other boxes of this 
      layer will be set to 0.
@end enumerate

@item wc_layer
Specify cloud properties for layers instead of levels (see also 
wc_file). If wc_layer is specified, cloud properties are 
assumed to be constant over the layer.
@item wc_properties
Define how liquid water content and effective droplet radius are translated 
to optical properties. Possible choices are
@table @code
@item hu 
Parameterization by Hu and Stamnes 
(An accurate parameterization of the radiative properties 
of water clouds suitable for use in climate models, J. Clim., 6, 728-742, 1993); 
this is the default setting. Note that 
the parameterization is somewhat different for 'correlated_k FU' than for all other 
cases because in the latter case the parameterization from the newer (March 2000) 
Fu and Liou code is used while otherwise the data are taken from the original 
Hu and Stamnes paper. 
@item mie
Use pre-calculated Mie tables; the tables are expected in 
@code{data_files_path}/correlated_k/.../
@item filename
Read optical properties from specified filename; file format is as produced 
by Frank Evans' cloudprp; for each of the internal (computational) wavelengths, 
a separate file is expected. Use only if you really know what you are doing
(THIS OPTION IS SUBJECT TO CHANGE).
@end table
@item wc_scale_gg
Scale the water cloud asymmetry factor for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item wc_scale_ssa
Scale the water cloud single scattering albedo for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item wc_set_gg
Set the water cloud asymmetry factor for all wavelengths and altitudes
to a float between -1.0 and 1.0. This option is useful only for monochromatic 
calculations or in wavelength regions where the optical properties of water clouds 
can be considered constant, e.g. the ultraviolet range.
@item wc_set_ssa
Set the water cloud single scattering albedo for all wavelengths
and altitudes to a float between 0.0 and 1.0. This option is useful only for monochromatic 
calculations or in wavelength regions where the optical properties of water clouds 
can be considered constant, e.g. the ultraviolet range.
@item wc_set_tau
Set the total water cloud optical depth to a constant value for all wavelengths.
This option is useful only for monochromatic calculations or in wavelength regions 
where the optical properties of water clouds can be considered constant, e.g. the 
ultraviolet range.
@item wvn
Set the wavelength range by specifying first and last wavelength in nm. 
The default output wavelength grid is that defined in @code{solar_file}, 
unless @code{spline} is specified. Note that the radiative transfer calculations 
are done on an internal grid which can be influenced with @code{transmittance_wl_file}.
@item zout 
Output altitudes in km. One or more altitudes may be specified in increasing
magnitude. Output altitudes must be within the range defined in the
@code{atmosphere_file}. Note that @code{zout} does not restructure the
atmosphere model. Hence, if you specify @code{zout 0.730} and have your
atmosphere model in @code{atmosphere_file} go all the way down to 
sea level, i.e. 0.0km., output is presented at 0.730km and calculations
performed with an atmosphere between 0.0 and 0.730 km (and above of course). 
If you want calculations done for e.g. a mountain site you have to restructure
the atmosphere model and make sure it stops at the appropriate altitude.
This you may either due by editing the atmosphere file or by using 
@code{altitude}. IMPORTANT NOTE: Some of the solvers (disort, disort2, twostr) 
have rounding problems: uvspec may crash with an error message 
****  Input variable  UTAU  in error  **** when zout coincides 
with the lower or upper boundary of the atmosphere. If that happens, try to select 
a slightly different @code{zout} within the allowed range 
(e.g. @code{zout 0.001} instead of @code{zout 0.0}).
@end vtable
  @c50_01@ */

  /*  @50_02c@                
@cindex output, uvspec
@cindex uvspec, output
The uvspec output differs for the different solvers and is currently fixed 
for each solver. We hope to make it user controllable in the future.

@node DISORT and SDISORT
@subsubsection DISORT and SDISORT
For the disort and sdisort solvers @code{uvspec} outputs one block of data to 
standard output (stdout) for each wavelength. The format of the block is

@example
lambda rfldir rfldn flup uavgso uavgdn uavgup
@end example

@noindent if @code{umu} is not specified. If @code{umu} is specified the format
of the block is

@example
lambda rfldir rfldn flup uavgso uavgdn uavgup 
umu(0) u0u(umu(0)) 
umu(1) u0u(umu(1)) 
  .     .          
  .     .          
@end example

@noindent If both @code{umu} and @code{phi} is specified
the output format of each block is

@example
lambda rfldir rfldn flup uavgso uavgdn uavgup
                            phi(0)     ...        phi(m)
umu(0) u0u(umu(0))   uu(umu(0),phi(0)) ... uu(umu(0),phi(m))
umu(1) u0u(umu(1))   uu(umu(1),phi(0)) ... uu(umu(1),phi(m))
  .     .             .                     .
  .     .             .                     .
umu(n) u0u(umu(n))   uu(umu(n),phi(0)) ... uu(umu(n),phi(m))
@end example

@noindent and so on for each wavelength.

@node TWOSTR
@subsubsection TWOSTR
The format of the output line for the twostr solver is

@example
lambda rfldir rfldn flup uavg
@end example

for each wavelength.

@node POLRADTRAN
@subsubsection POLRADTRAN
The output from the @code{polradtran} solver depends on the number of
Stokes parameters, @code{polradtran_nstokes}. For each wavelength the 
output block is

@example
lambda down_flux(1) up_flux(1) ...  down_flux(is) up_flux(is)
@end example

@noindent if @code{phi} is not specified. Here @code{is} is the number
of Stokes parameters specified by @code{polradtran_nstokes}. If 
@code{phi} is specified the block is

@example
lambda down_flux(1) up_flux(1) ...  down_flux(is) up_flux(is)
                             phi(0)                 ...                 phi(m)
Stokes vector I
cmu(0)  uu_down(cmu(0),phi(0)) uu_up(cmu(0),phi(0)) ... uu_up(cmu(0),phi(m)) uu_up(cmu(0),phi(m))
cmu(1)  uu_down(cmu(1),phi(0)) uu_up(cmu(1),phi(0)) ... uu_up(cmu(1),phi(m)) uu_up(cmu(1),phi(m))
  .       .                                          .	                    
  .       .                                          .	                    
cmu(n)  uu_down(cmu(n),phi(0)) uu_up(cmu(n),phi(0)) ... uu_up(cmu(n),phi(m)) uu_up(cmu(n),phi(m))
Stokes vector Q
  .       .                                          .	                    
  .       .                                          .	                    
@end example

@noindent Note that @code{polradtran} outputs the total (=direct+diffuse) 
downward flux.

@node Description of symbols
@subsubsection Description of symbols

In the above output blocks the symbols used have the following meaning. 

@vtable @code
@item cmu 
Computational polar angles from @code{polradtran}.
@item down_flux, up_flux
The total (direct+diffuse) downward (down_flux) and upward (up_flux) irradiances.
Same units as extraterrestrial irradiance.
@item lambda
Wavelength (nm)
@item rfldir
Direct beam irradiance (same units as extraterrestrial irradiance, e.g mW/(m2 nm) if 
using the @file{atlas3} spectrum in the @file{data/solar_flux} directory.)
@item rfldn
Diffuse down irradiance, i.e. total minus direct beam (same units
as @code{rfldir}).
@item flup
Diffuse up irradiance (same units as @code{rfldir}).
@item uavg
The mean intensity. Proportional to the actinic flux. (same units as @code{rfldir}).
@item uavgso
Direct beam contribution to the mean intensity. (same units as @code{rfldir}).
@item uavgdn
Diffuse downward radiation contribution to the mean intensity. (same units as @code{rfldir}).
@item uavgup
Diffuse upward radiation contribution to the mean intensity. (same units as @code{rfldir}).
@item u0u
The azimuthally averaged intensity at @code{numu} user specified 
angles @code{umu}. (units of e.g. mW/(m2 nm sr) if using the @file{atlas3} spectrum 
in the @file{data/solar_flux} directory.)  
@item uu
The radiance (intensity) at @code{umu} and @code{phi} user specified 
angles. (units of e.g. mW/(m2 nm sr) if using the @file{atlas3} spectrum 
in the @file{data/solar_flux} directory.)  
@item uu_down, uu_up
The downwelling and upwelling radiances (intensity) at @code{cmu} and 
@code{phi} angles. (units of e.g. mW/(m2 nm sr) if using the @file{atlas3} spectrum 
in the @file{data/solar_flux} directory.)  

@end vtable

@noindent The total downward irradiance is given by

@example
irr_down = rfldir + rfldn
@end example

@noindent The total mean intensity is given by

@example
uavg = uavgso + uavgdn + uavgup
@end example

@noindent If @code{deltam} is on it does not make sense to look at the individual 
contributions to @code{uavg} since they are delta-M scaled.

  @c50_02@ */

  /*  @50_03c@                

An example of a complete input file for a clear sky atmosphere is

@example
@include ../examples/UVSPEC_CLEAR.INP
@end example

@noindent 
The atmosphere model, i.e. pressure, temperature, and ozone concentrations 
profiles are read from  the file @code{../data/atmmod/afglus.dat}. The 
extraterrestrial solar flux is read from the file 
@code{../data/solar_flux/atlas_plus_modtran}.

A wavelength dependent surface albedo may be specified by using @code{albedo_file}
instead of @code{albedo}.

For simulations of measured spectra the change in solar zenith angle during a 
scan must be accounted for. This is done by using @code{sza_file} instead of
@code{sza}. 

Aerosol information is usually of importance when simulating measured spectra.
Aerosols may be specified in a hierarchical way. The most simple way to define 
an aerosol is by the command @code{aerosol_default} which will set up the
aerosol model by
E.P. Shettle, "Models of aerosols, clouds and precipitation for 
atmospheric propagation studies", in "Atmospheric propagation in the 
uv, visible, ir and mm-region and related system aspects", 
AGARD Conference Proceedings (454), 1989. The default properties are 
a rural type aerosol in the boundary layer, background aerosol above 2km,
spring-summer conditions and a visibility of 50km. These settings may
be modified with @code{aerosol_haze}, @code{aerosol_vulcan}, @code{aerosol_season},
and @code{aerosol_visibility}. More information can be introduced step by step,
overwriting the default parameters. @code{aerosol_tau_file}, @code{aerosol_ssa_file},
and @code{aerosol_gg_file}, can be used to define the profiles of optical thickness,
single scattering albedo, and asymmetry parameter. The integrated optical thickness 
can be set to a constant value using @code{aerosol_set_tau}.
The single scattering albedo may be scaled by @code{aerosol_scale_ssa} and 
set by @code{aerosol_set_ssa}.  
The aerosol asymmetry factor may be set by @code{aerosol_set_gg}. 
The wavelength dependence of the aerosol optical depth is specified using the 
@code{aerosol_angstrom} parameter. @code{aerosol_moments_file} allows specification of 
the scattering phase function. If microphysical properties are available these 
may be introduced by defining the complex index of refraction @code{aerosol_refrac_index} 
or @code{aerosol_refrac_file} and the size distribution @code{aerosol_sizedist_file}. 
Especially the last parameters are work in progress and therefore by no means complete. 

The following list is an overview of the aerosol description parameters. The 
entries are arranged in a way that a parameter 'overwrites' all values higher up 
in the list.

@table @code
@item aerosol_default
Generate default aerosol according to Shettle (1989)
@item aerosol_vulcan, aerosol_haze, aerosol_season, aerosol_visibility
Set Shettle (1989) aerosol properties (aerosol type, visibility)
@item aerosol_tau_file, aerosol_ssa_file, aerosol_gg_file
Overwrite profiles of optical thickness, single scattering albedo, and
asymmetry parameter
@item aerosol_moments_file
Specify a phase function to be used instead of the Henyey-Greenstein phase function
@item aerosol_refrac_index, aerosol_refrac file, aerosol_sizedist_file
Calculate optical properties from size distribution and index of refraction using 
Mie theory. Here is an exception from the rule that ALL values defined above are 
overwritten because the optical thickness profile is re-scaled so that the 
optical thickness at the first internal wavelength is unchanged. It is done 
that way to give the user an easy means of scaling the specifying the optical 
thickness at a given wavelength. 
@item aerosol_set_gg, aerosol_set_ssa, aerosol_scale_ssa, aerosol_set_tau
Overwrite profiles of asymmetry parameter and single scattering albedo
@item aerosol_angstrom
Overwrite the profile of the optical thickness.
@end table

An example for a uvspec aerosol description is

@example
@include ../examples/UVSPEC_AEROSOL.INP
@end example

@noindent 
By combining this with the clear sky example given above a complete uvspec
input file including aerosol is constructed.

To calculate radiances the following lines will do the job when combined
with the e.g. the clear sky example above

@example
@include ../examples/UVSPEC_RADIANCES.INP
@end example

Water clouds may be included by specifying @code{wc_file}.

@example
@include ../examples/UVSPEC_WC.INP
@end example

@noindent 
The file @code{WC.DAT} is found in the examples directory.

A very simple but complete example for a correlated-k approximation 
of the solar spectrum:

@example
@include ../examples/UVSPEC_KATO.INP
@end example

Here, the solar spectrum is split up into 32 bands according to 
Kato et al. (1999). In order to calculate integrated shortwave irradiance,
remove the 'transmittance' line and sum the outputs.

More examples with output are found in the @file{examples} directory.

      @c50_03@   */

  /***************** End of documentation *********************/

  int i=0, c=0;
  int status=0;

  char base[255]="";
  
  /* Initialization */

  /* allocate memory for filenames */
  Input.filename         = calloc (FN_NN,     sizeof(char *));
  Input.aer.filename     = calloc (FN_AER_NN, sizeof(char *));
  Input.rte.mc.filename  = calloc (FN_MC_NN,  sizeof(char *));


  for (i=0; i<FN_NN; i++)
    Input.filename[i]     = calloc(FILENAME_MAX+1, sizeof(char));
  
  for (i=0; i<FN_AER_NN; i++) 
    Input.aer.filename[i] = calloc(FILENAME_MAX+1, sizeof(char));

  for (i=0; i<FN_MC_NN; i++)
    Input.rte.mc.filename[i] = calloc(FILENAME_MAX+1, sizeof(char));


  Input.flexstor          = 0;
  Input.header            = 0;
  Input.source            = SRC_SOLAR;  /* default: solar source     */
  Input.bandwidth         = 1.0;        /* default bandwidth in cm-1 */
  Output.print.phi        = 1;          /* Default print radiances   */
  Output.atm.ndens        = 6;   /* Default value for atmospheric files with   */
			         /*  air, o3, o2, h2o, co2, and no2 densities. */
  Input.ck_scheme         = CK_NONE;
  Input.ck_h2ocont        = 1;
  Input.cloudcover        = -999;
  Input.ipa               = 0;
  Input.convolve          = 0;
  Input.spline            = 0;
  Input.crs.o3_type       = O3_MOLINA;
  Input.crs.filename      = "";
  Input.day_of_year       = -999;
  Input.ozone_column      = -999;
  Input.pressure          = -999;
  Input.temperature       = -999;

  Input.aer.alpha         = 0.0;
  Input.aer.beta          = -999.;
  Input.aer.haze          = 0;
  Input.aer.scale_ssa     = -999.;
  Input.aer.set_gg        = -999.;
  Input.aer.set_tau       = -999.;
  Input.aer.set_ssa       = -1.;
  Input.aer.seasn         = 0;
  Input.aer.visib         = -999.;
  Input.aer.vulcan        = 0;

  Input.atm.albedo        = 0.0;
  Input.atm.altitude      = 0.0;
  Input.atm.sza           = 0.0;

  Input.transmittance     = 0;
  Input.rte.fisot         = 0;
  Input.rte.maxphi        = 0;
  Input.rte.nscat         = 20;
  Input.rte.nstr          = 6;
  Input.rte.solver        = DISORT;
  Input.rte.pol_stokes[0] = 'I';
  Input.rte.pol_stokes[1] = 'Q';
  Input.rte.pol_stokes[2] = 'U';
  Input.rte.pol_stokes[3] = 'V';

  Input.rte.mc.absorption = MCABS_NONE;
  Input.rte.mc.photons    = 100000;
  Input.rte.mc.momcld     = MOMCLD_NONE;
  Input.rte.mc.Nx_sample  = 0;
  Input.rte.mc.Ny_sample  = 0;
  Input.rte.mc.Nx_cloud   = 0;
  Input.rte.mc.Ny_cloud   = 0;
  Input.rte.mc.Nz_cloud   = 0;
  Input.rte.mc.passback3D = 0;
  Input.rte.mc.ipa        = 0;

  Input.wl.wl_start       = -999;
  Input.wl.wl_end         = -999;
  Input.rte.nphi          = 0;
  Input.rte.numu          = 0;
  Input.rte.nprndis       = 0;
  Input.rte.deltam        = 1;
  Input.rte.pol_aziorder  = 0;
  Input.rte.pol_max_delta_tau = 1e-5;
  Input.rte.pol_nstokes   = 1;
  Input.rte.pol_quad_type = "G";
  Input.rte.pol_src_code  = 1;
  Input.rte.reverse       = 0;
  Input.atm.nzout         = 0;
  Input.wc.tau            = -999;
  Input.wc.gg             = -999;
  Input.wc.ssa            = -999;
  Input.wc.scale_gg       = -999;
  Input.wc.scale_ssa      = -999;
  Input.wc.layer          = 0;
  Input.wc.properties     = CLDPROP_DEFAULT;
  Input.ic.tau            = -999;
  Input.ic.gg             = -999;
  Input.ic.ssa            = -999;
  Input.ic.scale_gg       = -999;
  Input.ic.scale_ssa      = -999;
  Input.ic.layer          = 0;
  Input.ic.properties     = CLDPROP_DEFAULT;
  Input.absorption        = 1;
  Input.molabs            = MOLABS_CALC;
  Input.rayleigh          = RAYLEIGH_CALC;
  Input.mixing_ratio = calloc(MX_NN, sizeof(float));
  for (i=0; i<MX_NN; i++)
    Input.mixing_ratio[i] = -999;

  while ((c=getopt (argc, argv, "h")) != EOF)  {
    switch(c)  {
    case 'h': 
      usage();
      return (-1);
      break;
    default:
      usage();
      return (-1);
    }
  }

  /* Read the input file */

  yylex();


  /* number of phase function moments equals the number of streams */
  /* per default, but may be changed later in the code             */
  Output.atm.nmom = Input.rte.nstr;      


  if (strlen(Input.crs.filename) > 0) {
      /* -3 because minus z, p and T column */
      Output.atm.ndens    = Input.crs.column - 3 + 1;
  }

  if (Input.rte.nphi <= 0) {
    switch (Input.rte.solver) {
    case SDISORT:
    case DISORT:
    case DISORT2:
      Input.rte.maxphi = 3;  /* Minimum number required by disort. */
      Input.rte.nphi   = 1;
      Input.rte.phi    = (float *) calloc (Input.rte.maxphi, sizeof(float));
      Output.print.phi = 0;  /* Do not print radiances if user do not want them */
      break;
    case POLRADTRAN:
      if (Input.rte.pol_aziorder == 0) 
	Input.rte.nphi = 0;
      break;
    }
    /* Input.rte.maxphi = 3; */ /* Minimum number required by disort. */
    /* Input.rte.nphi   = 1; */
    /* Input.rte.phi    = (float *) calloc (Input.rte.maxphi, sizeof(float)); */
  }

  if (Input.rte.numu <= 0) {
    Input.rte.maxumu = Input.rte.nstr;
    Input.rte.umu    = (float *) calloc (Input.rte.maxumu, sizeof(float));
  }

  if (Input.atm.nzout == 0) {
    Input.atm.nzout   = 1;
    Input.atm.zout    = (float *) calloc (Input.atm.nzout, sizeof(float));

    if (Input.rte.solver == MONTECARLO)
      Input.atm.zout[0] = -999.0;
    else 
      Input.atm.zout[0] = 0.0;
  }


  /* surface temperature */
  Output.surface_temperature = Input.temperature;


  /* no user-defined transmittance grid if correlated-k */
  if (Input.ck_scheme!=CK_NONE && strlen(Input.filename[FN_WLTRANS])>0) {
    strcpy (Input.filename[FN_WLTRANS], "");
    fprintf (stderr, " ... option transmittance_wl_file ignored\n");
    fprintf (stderr, " ... (does not make sense with correlated-k)\n");
  }


  /* Default MYSTIC output filename */
  if (strlen(Input.rte.mc.filename[FN_MC_BASENAME]) == 0)
    strcpy (Input.rte.mc.filename[FN_MC_BASENAME], "mc");
  

  /* MYSTIC 3D fields passed back to uvspec ? */
  if (Input.atm.nzout>0 && Input.rte.mc.Nx_sample>0 && Input.rte.mc.Ny_sample>0 
      && Input.rte.solver==MONTECARLO)
    Input.rte.mc.passback3D=1;

  if (strlen(Input.rte.mc.filename[FN_MC_MOMCLD])>0 && strlen(Input.rte.mc.filename[FN_MC_PCLD])>0) {
    fprintf (stderr, "Error. Please specify either mc_momcld_file or mc_pcld_file but\n");
    fprintf (stderr, "not both at the same time!\n");
    return -1;
  }


  if (Input.rte.mc.momcld==MOMCLD_DEFAULT) {
    switch (Input.ck_scheme) {
    case CK_KATO:
      strcpy (Input.rte.mc.filename[FN_MC_MOMTRNC], Input.filename[FN_PATH]);
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], "/correlated_k/kato/kato.");
      break;

    case CK_FU:
      strcpy (Input.rte.mc.filename[FN_MC_MOMTRNC], Input.filename[FN_PATH]);
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], "/correlated_k/fu/fu.");
      break;
      
    case CK_AVHRR_KRATZ:
      fprintf (stderr, "Error, default phase function files for 'correlated_k AVHRR_KRATZ'\n");
      fprintf (stderr, "       have not yet been created!\n");
      return -1;
      break;

    case CK_NONE:
      fprintf (stderr, "Error, 'mc_momcld_file default' makes only sense together with\n");
      fprintf (stderr, "       'correlated_k'\n");
      return -1;

    default:
      fprintf (stderr, "Error, unknown correlated_k scheme %d\n", Input.ck_scheme);
      return -1;
    }
  }
	     

  switch (Input.wc.properties) {
  case CLDPROP_MIE:
  case CLDPROP_FILE:
    if (Input.rte.solver==DISORT) {
      fprintf (stderr, "\n");
      fprintf (stderr, "*** Warning: You specified a tabulated phase function for water clouds.\n");
      fprintf (stderr, "***  It is strongly recommended to use DISORT2 in this case because\n");
      fprintf (stderr, "***  the older DISORT version uses only the first NSTR moments of the\n");
      fprintf (stderr, "***  phase function which might result in really bad distortions.\n");
      fprintf (stderr, "***  DISORT2 consideres all moments correctly.\n");
      fprintf (stderr, "\n");
    }
    break;
  default:
    break;
  }


  switch (Input.ic.properties) {
  case CLDPROP_MIE:
  case CLDPROP_FILE:
    if (Input.rte.solver==DISORT) {
      fprintf (stderr, "\n");
      fprintf (stderr, "*** Warning: You specified a tabulated phase function for ice clouds.\n");
      fprintf (stderr, "*** It is strongly recommended to use DISORT2 in this case because\n");
      fprintf (stderr, "*** the older DISORT version uses only the first NSTR moments of the\n");
      fprintf (stderr, "*** phase function which might result in really bad distortions.\n");
      fprintf (stderr, "*** DISORT2 consideres all moments correctly");
      fprintf (stderr, "\n");
    }
    break;
  default:
    break;
  }


  /* water cloud properties */
  if (Input.wc.properties == CLDPROP_MIE) {
    switch (Input.ck_scheme) {
    case CK_KATO:
      strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_WCLOUD_PROP], "/correlated_k/kato/");
      strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      break;	    
		    
    case CK_FU:	    
      strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_WCLOUD_PROP], "/correlated_k/fu/");
      strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      break;
      
    case CK_AVHRR_KRATZ:
      strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_WCLOUD_PROP], "/correlated_k/kratz/");
      strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      break;

    case CK_NONE:
      fprintf (stderr, "Error, 'wc_properties' only implemented for correlated-k!\n");
      return -1;

    default:
      fprintf (stderr, "Error, unknown correlated_k scheme %d\n", Input.ck_scheme);
      return -1;
    }

    fprintf (stderr, " ... reading water cloud optical properties from %s*\n", 
	     Input.filename[FN_WCLOUD_PROP]);
  }
  


  /* ice cloud properties */
  if (Input.ic.properties == CLDPROP_MIE) {

    switch (Input.ck_scheme) {
    case CK_KATO:
      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_ICLOUD_PROP], "/correlated_k/kato/");
      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      break;	    
		    
    case CK_FU:	    
      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_ICLOUD_PROP], "/correlated_k/fu/");
      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      break;
      
    case CK_AVHRR_KRATZ:
      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_ICLOUD_PROP], "/correlated_k/kratz/");
      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      break;

    case CK_NONE:
      fprintf (stderr, "Error, 'ic_properties' only implemented for correlated-k!\n");
      return -1;

    default:
      fprintf (stderr, "Error, unknown correlated_k scheme %d\n", Input.ck_scheme);
      return -1;
    }

    fprintf (stderr, " ... reading ice cloud optical properties from %s*\n", 
	     Input.filename[FN_ICLOUD_PROP]);
  }
    


  if (Input.rte.mc.momcld==MOMCLD_WATER || Input.rte.mc.momcld==MOMCLD_ICE) {

    switch (Input.rte.mc.momcld) {
    case MOMCLD_WATER:
      strcpy (base, "wc.");
      break;
    case MOMCLD_ICE:
      strcpy (base, "ic.");
      break;
    default:
      break;
    }

    switch (Input.ck_scheme) {
    case CK_KATO:
      strcpy (Input.rte.mc.filename[FN_MC_MOMTRNC], Input.filename[FN_PATH]);
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], "/correlated_k/kato/");
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], base);
      break;

    case CK_FU:
      strcpy (Input.rte.mc.filename[FN_MC_MOMTRNC], Input.filename[FN_PATH]);
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], "/correlated_k/fu/");
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], base);
      break;
      
    case CK_AVHRR_KRATZ:
      fprintf (stderr, "Error, default phase function files for 'correlated_k AVHRR_KRATZ'\n");
      fprintf (stderr, "       have not yet been created\n");
      return -1;
      break;

    case CK_NONE:
      fprintf (stderr, "Error, 'mc_momcld_file default' makes only sense together with\n");
      fprintf (stderr, "       'correlated_k'\n");
      return -1;

    default:
      fprintf (stderr, "Error, unknown correlated_k scheme %d\n", Input.ck_scheme);
      return -1;
    }
  }
	     


    

  /********************************/
  /* Do the radiation calculation */
  /********************************/
  
  status = uvspec(Input,&Output);

  if (status!=0) {
    fprintf (stderr, "Error %d during execution of uvspec\n", status);
    return status;
  }
  

  /****************************************************************/
  /* Print output ***FIXME*** THIS SHOULD BE USER DEFINED SOMEHOW */
  /****************************************************************/

  print_output(Input, &Output);

  free_output(Input, &Output);
  return status;
}



static int print_output(input_struct Input, output_struct *output) 
{
  int j=0, is=0, iu=0, iv=0, lev=0;

  if (Input.flexstor) {
    printf("flexstor 1 0 suvdama 1 1\n");
    printf("# This flexstor output has not been checked by flxcheck.\n");
    printf("# Hence it may not follow flexstor/suvdama rules. This will\n");
    printf("# be improved soon.\n");
    printf("model_name() uvspec, version %4.2f\n", VERSION);
    if ( fabs(Input.aer.beta-0.0) > 0.0 ) {
      printf("angstrom_alpha()  %f\n", Input.aer.alpha);
      printf("angstrom_beta()  %f\n", Input.aer.beta);
    }
    printf("ozone_column(DU) %f\n", output->ozone_column);
    printf("pressure(mb) %f\n", Input.pressure);
    printf("self_day_of_year() %d\n", Input.day_of_year);
    
    /*
    printf("self_zout(km)");
    for (lev=0;lev<Input.atm.nzout;lev++)
      printf("  %f", Input.atm.zout[lev]);
    printf("\n");
    */

    printf("self_temper(K)");
    for (lev=0;lev<Input.atm.nzout;lev++)
      printf("  %f", output->atm.temper_out[lev]);
    
    printf("\n");
    printf("self_day_of_year() %d\n", Input.day_of_year);
    printf("self_eccentricity_factor() %f\n", output->eccentricity_factor);
    printf("begin table data\n");
    for (lev=0;lev<Input.atm.nzout;lev++) {
      printf("WL(nm) SZA(deg.dec) self_rfldir(mW m-2 nm-2) self_rfldn(mW m-2 nm-2) self_rflup(mW m-2 nm-2)");
      printf(" self_uavgso(mW m-2 nm-2) self_uavgdn(mW m-2 nm-2) self_uavgup(mW m-2 nm-2) ");
    }
    printf("\n");
    for (iv=0;iv<output->wl.nlambda_h;iv++) {
      for (lev=0;lev<Input.atm.nzout;lev++) {
	if ( Input.rte.solver == TWOSTR ) {
	  printf("%7.3f %13.6e %13.6e %13.6e %13.6e ", 
		 output->wl.lambda_h[iv], 
		 output->rfldir[lev][iv], 
		 output->rfldn[lev][iv], 
		 output->flup[lev][iv],
		 output->uavg[lev][iv]);
	}
	else {
	  printf("%7.3f %7.3f %8.5f %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e ",
		 output->wl.lambda_h[iv], 
		 output->sza[iv], 
		 Input.atm.zout[lev],
		 output->rfldir[lev][iv], 
		 output->rfldn[lev][iv], 
		 output->flup[lev][iv],
		 output->uavgso[lev][iv], 
		 output->uavgdn[lev][iv], 
		 output->uavgup[lev][iv]);
	}
	printf("\n");
      }
    }
    printf("end table data\n");
  }
  else {
    if (Input.header) {
      printf("#aerosol_haze                 : %16d\n",   Input.aer.haze);
      printf("#aerosol_season               : %16d\n",   Input.aer.seasn);
      printf("#aerosol_set_gg               : %16.3f\n", Input.aer.set_gg);
      printf("#aerosol_set_ssa              : %16.3f\n", Input.aer.set_ssa);
      printf("#aerosol_tau_file             : %s\n",     Input.aer.filename[FN_AER_TAU]);
      printf("#aerosol_ssa_file             : %s\n",     Input.aer.filename[FN_AER_SSA]);
      printf("#aerosol_gg_file              : %s\n",     Input.aer.filename[FN_AER_GG]);
      printf("#aerosol_moments_file         : %s\n",     Input.aer.filename[FN_AER_MOM]);
      printf("#aerosol_visibility           : %16.3f\n", Input.aer.visib);
      printf("#albedo                       : %16.3f\n", Input.atm.albedo);
      printf("#angstrom                     : %16.3f  %16.3f\n", Input.aer.alpha, Input.aer.beta);
      printf("#atmosphere_file              : %s\n", Input.filename[FN_ATMOSPHERE]);
      printf("#day_of_year                  : %16d\n", Input.day_of_year);
      switch (Input.rte.deltam) {
      case 0:
	printf("#deltam                       : off\n");
	break;
      case 1:
	printf("#deltam                       : on\n");
	break;
      default:
	printf("#deltam                       : unknown\n");
	break;
      }

      switch (Input.crs.o3_type) {
      case O3_BASSPAUR:
	printf("#o3_crs                       : Bass_and_Paur\n");
	break;
      case O3_MOLINA:
	printf("#o3_crs                       : Molina\n");
	break;
      case O3_DAUMONT:
	printf("#o3_crs                       : Daumont\n");
	break;
      default:
	printf("#o3_crs                       : unknown\n");
	break;
      }

      printf("#ozone_column                 : %16.3f\n", Input.ozone_column);

      switch (Input.rte.solver) {
      case SDISORT:
	printf("#rte_solver                   : sdisort\n");
	break;
      case DISORT:
	printf("#rte_solver                   : disort\n");
	break;
      case DISORT2:
	printf("#rte_solver                   : disort2\n");
	break;
      case MONTECARLO:
	printf("#rte_solver                   : mystic\n");
	break;
      case POLRADTRAN:
	printf("#rte_solver                   : polradtran\n");
	break;
      case SUCORD:
	printf("#rte_solver                   : sucorcsca\n");
	break;
      case TWOSTR: 
	printf("#rte_solver                   : twostr\n");
	break;
      default:
	printf("#rte_solver                   : unknown\n");
	break;
      }

      printf("#slit_function_file           : %s\n", 
	     Input.filename[FN_SLITFUNCTION]);
      printf("#solar_file                   : %s\n", 
	     Input.filename[FN_EXTRATERRESTRIAL]);
      printf("#wvn                          : %16.3f  %16.3f\n", 
	     Input.wl.wl_start, Input.wl.wl_end);
      printf("#zout                         : ");
      for (lev=0;lev<Input.atm.nzout;lev++)
	printf(" %16.3f", Input.atm.zout[lev]);
      printf("\n");
    }

    if (Input.rte.solver == POLRADTRAN) {
      for (iv=0;iv<output->wl.nlambda_h;iv++) {
	for (lev=0;lev<Input.atm.nzout;lev++) {
	  printf("%7.3f ", output->wl.lambda_h[iv]);
	  for (is=0;is<Input.rte.pol_nstokes;is++)
	    printf("%13.6e %13.6e ",
		   output->down_flux[lev][is][iv], 
		   output->up_flux[lev][is][iv]);
	    printf("\n");

	  if ( output->print.phi && Input.rte.nphi > 0 ) {
	    for (j=0;j<Input.rte.nphi;j++) {
	      printf("            %13.3f   ", Input.rte.phi[j]);
	    }
	    printf("\n");
	    for (is=0;is<Input.rte.pol_nstokes;is++) {
	      printf("Stokes vector %c\n", Input.rte.pol_stokes[is]);
	      for (iu=0;iu<Input.rte.nstr/2;iu++) {
		printf("%7.4f ", output->mu_values[iu] );
		for (j=0;j<Input.rte.nphi;j++) {
		  printf("%13.6e %13.6e ", 
			 output->down_rad[lev][j][iu][is][iv], 
			 output->up_rad[lev][j][iu][is][iv]);
		}
		printf("\n");
	      }
	    }
	  }
	}
      }
    }
    else {
      for (iv=0;iv<output->wl.nlambda_h;iv++) {
	for (lev=0;lev<Input.atm.nzout;lev++) {
	  if (Input.rte.solver == TWOSTR) {
	    printf("%7.3f %13.6e %13.6e %13.6e %13.6e\n", 
		   output->wl.lambda_h[iv], 
		   output->rfldir[lev][iv], 
		   output->rfldn[lev][iv], 
		   output->flup[lev][iv],
		   output->uavg[lev][iv]);
	  }
	  else {
	    printf("%7.3f %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e\n", 
		   output->wl.lambda_h[iv], 
		   output->rfldir[lev][iv], 
		   output->rfldn[lev][iv], 
		   output->flup[lev][iv],
		   output->uavgso[lev][iv], 
		   output->uavgdn[lev][iv], 
		   output->uavgup[lev][iv]);
	  }

	  if ( Input.rte.numu > 0 && !output->print.phi ) {
	    for (iu=0;iu<Input.rte.numu;iu++)
	      printf("%7.4f %13.6e \n", Input.rte.umu[iu], output->u0u[lev][iu][iv]);
	  }
	  if ( output->print.phi && (Input.rte.solver == DISORT || 
				     Input.rte.solver == DISORT2 || Input.rte.solver == SDISORT)
	       && Input.rte.numu> 0) {
	    printf("                      ");
	    for (j=0;j<Input.rte.nphi;j++)
	      printf(" %13.3f ", Input.rte.phi[j]);
	    printf("\n");
	    for (iu=0;iu<Input.rte.numu;iu++) {
	      printf("%7.4f %13.6e ", Input.rte.umu[iu], output->u0u[lev][iu][iv]);
	      for (j=0;j<Input.rte.nphi;j++)
		printf(" %13.6e ", output->uu[lev][j][iu][iv]);
	      printf("\n");
	    }
	  } 
	}
      }    
    }
  }
  fflush(stdout);

  return 0;
}  
  



static void free_output(input_struct Input, output_struct *output) 
{
  int status=0;

  status = ASCII_free_float_3D (output->down_flux, 
				Input.atm.nzout, Input.rte.pol_nstokes);
  status = ASCII_free_float_3D (output->down_flux_t, 
				Input.atm.nzout, Input.rte.pol_nstokes);
  status = ASCII_free_float_3D (output->up_flux, 
				Input.atm.nzout, Input.rte.pol_nstokes);
  status = ASCII_free_float_3D (output->up_flux_t, 
				Input.atm.nzout, Input.rte.pol_nstokes);
  status = ASCII_free_float_5D (output->down_rad, 
				Input.atm.nzout, Input.rte.nphi, 
				Input.rte.nstr/2, Input.rte.pol_nstokes);
  status = ASCII_free_float_5D (output->down_rad_t, 
				Input.atm.nzout, Input.rte.nphi,
				Input.rte.nstr/2, Input.rte.pol_nstokes);
  status = ASCII_free_float_5D (output->up_rad, 
				Input.atm.nzout, Input.rte.nphi,
				Input.rte.nstr/2, Input.rte.pol_nstokes);
  status = ASCII_free_float_5D (output->up_rad_t, 
				Input.atm.nzout, Input.rte.nphi,
				Input.rte.nstr/2, Input.rte.pol_nstokes);

  free(output->dtaer);
  free(output->dtair);
  free(output->dtauc);
  free(output->dtcld);
  free(output->dtice);
  free(output->dtmol);
  free(output->mu_values);
  free(output->omaer);
  free(output->omcld);
  free(output->omice);
  free(output->sza);
}


static float yytext2float(char* yytext, int which_token, int max_token) {
  double dnumber=0;
  char token[]=" ";
  char *p=NULL, *s=NULL;
  int count=0;
  
  p = strtok(yytext,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }
  
  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  
  dnumber = atof(s);
  return (float) dnumber;
}


static double yytext2double(char* yytext, int which_token, int max_token) {
  double dnumber=0;
  char token[]=" ";
  char *p=NULL, *s=NULL;
  int count=0;
  
  p = strtok(yytext,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }
  
  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  
  dnumber = atof(s);
  return dnumber;
}


static float* yytext2floats(char* yytext, int nary) {
  int i=0;
  char token[]=" ";
  char *p=NULL;
  
  /* Fill up ary */
  float *ary = (float *) calloc (nary, sizeof(float));
  
  p = strtok(yytext,token);
  i = 0;

  while ((p = strtok(NULL,token)) != NULL)
    ary[i++] = atof(p);
  
  return ary;
}


static int yytext2nfloats(char* yytext) {
  int nary=0;
  char token[]=" ";
  char *p=NULL;

  /* Get number of ary elements */
  nary = 0;
  p = strtok(yytext,token);

  while ( (p = strtok(NULL,token)) != NULL)
    nary++;

  return nary;
}


static int* yytext2integers(char* yytext, int nary) {
  int i=0;
  char token[]=" ";
  char *p=NULL;

  /* Fill up ary */
  int *ary    = (int *) calloc (nary, sizeof(int));

  p = strtok(yytext,token);
  i = 0;

  while ((p = strtok(NULL,token)) != NULL)
    ary[i++] = atoi(p);

  return ary;
}


static int yytext2nintegers(char* yytext) {
  int nary=0;
  char token[]=" ";
  char *p=NULL;

  /* Get number of ary elements */
  nary = 0;
  p = strtok(yytext,token);

  while ((p = strtok(NULL,token)) != NULL) 
    nary++;

  return nary;
}


static int yytext2int(char* yytext, int which_token, int max_token) {
  int inumber=0;
  char token[]=" ";
  char *p=NULL, *s=NULL;
  int count=0;

  p = strtok(yytext,token);
  count = 1;
  while ( (p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  inumber = atoi(s);
  
  return inumber;
}


static char* yytext2string(char* yytext, int which_token, int max_token) {
  char token[]=" ";
  char *p=NULL, *s=NULL, *string=NULL;
  int count=0;

  p = strtok(yytext,token);
  count = 1;
  while ( (p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  string = (char *) calloc (strlen(s)+1, sizeof (char));
  strcpy (string, s);
  return string;
}


/* like yytext2string, but string is copied to a pre-allocated string dest */
static int yytextCstring(char *dest, char* yytext, int which_token, int max_token) {
  char token[]=" ";
  char *p=NULL, *s=NULL;
  int count=0;

  p = strtok(yytext,token);
  count = 1;
  while ( (p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  strcpy (dest, s);
  return 0;
}


